/*
 *  Externalization Service for MICO
 *  Copyright (C) 1999 Dmitry V. Sedov
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 *  or to my private e-mail:
 *                 sedov@postman.simcb.ru
 */


#include <mico/InterfaceDefsHelper.h>
#include <mico/NamedRoleTypesHelper.h>
#include <mico/RoleFactory_impl.h>
#include <mico/RelationshipFactory_impl.h>
#include <mico/ExternalizationReference_impl.h>
#include <mico/CosTrading.h>
#include <mico/CosTradingRepos.h>

#ifdef USE_CORBA_2_1
#define CORBA_CXX_PREFIX(x) _##x
#else
#define CORBA_CXX_PREFIX(x) _cxx_##x
#endif


static void
usage(const char* progName)
{
	cerr << "Usage:\n";
	cerr << progName << " [options]\n"
	"\n"
	"Options:\n"
	"-h, --help                Show this message.\n"
	"-v, --version             Show CosExternalizationReference Service Factories version.\n"
	"-t, --trading             Try to resolve Trading Service and export to it.\n"
	;
}


int main( int argc, char *argv[] )
{
    const char* progName = argv[0];
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );

    CORBA::Boolean trader = false;
    CORBA::Long i;
    for(i = 1 ; i < argc && *argv[i] == '-' ; i++)
    {
	if(strcmp(argv[i], "--help") == 0 ||
	    strcmp(argv[i], "-h") == 0)
	{
	    usage(progName);
	    return 0;
	}
	else if(strcmp(argv[i], "--version") == 0 ||
		strcmp(argv[i], "-v") == 0)
	{
	    cerr << "Version of CosExternalizationContainment Service Factories is " << MICO_VERSION << endl;
	    return 0;
	}
	else if(strcmp(argv[i], "--trading") == 0 ||
		strcmp(argv[i], "-t") == 0)
	{
	    trader = true;
	}
	else
	{
	    cerr << progName << ": unknown option `"
		 << argv[i] << "'\n" << endl;
	    usage(progName);
	    return 1;
	}
    }

    CORBA::Object_var poaobj = orb->resolve_initial_references ("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow (poaobj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();

    InterfaceDefsHelper* idefs_references = new InterfaceDefsHelper (orb);
    idefs_references->add ("IDL:omg.org/CosCompoundExternalization/Node:1.0");

    InterfaceDefsHelper* idefs_referencedby = new InterfaceDefsHelper (orb);
    idefs_referencedby->add ("IDL:omg.org/CosCompoundExternalization/Node:1.0");


    RoleFactory_impl<ExternalizationReferencesRole_impl>* references_role_factory 
	= new RoleFactory_impl<ExternalizationReferencesRole_impl>;

    references_role_factory->set_max_cardinality (4294967295ul);
    references_role_factory->set_related_object_types(idefs_references->get_interfaces ());
    references_role_factory->set_orb (orb);

    RoleFactory_impl<ExternalizationReferencedByRole_impl>* referencedby_role_factory 
	= new RoleFactory_impl<ExternalizationReferencedByRole_impl>;

    referencedby_role_factory->set_max_cardinality (4294967295ul);
    referencedby_role_factory->set_related_object_types(idefs_referencedby->get_interfaces ());
    referencedby_role_factory->set_orb (orb);
  

    PortableServer::ObjectId_var references_id 
	= PortableServer::string_to_ObjectId ("ExternalizationReferencesRole_impl");
    PortableServer::ObjectId_var referencedby_id 
	= PortableServer::string_to_ObjectId ("ExternalizationReferencedByRole_impl");

    NamedRoleTypesHelper*
	nrth_reference = new NamedRoleTypesHelper (orb);
    nrth_reference->add ("IDL:omg.org/CosExternalizationReference/ReferencesRole:1.0", 
			 "ExternalizationReferencesRole");
    nrth_reference->add ("IDL:omg.org/CosExternalizationReference/ReferencedByRole:1.0", 
			 "ExternalizationReferencedByRole");
  

    RelationshipFactory_impl<ExternalizationReference_impl>* reference_factory 
	= new RelationshipFactory_impl<ExternalizationReference_impl> (orb);
  
    reference_factory->set_orb (orb);
    reference_factory->set_named_role_types (nrth_reference->get_named_role_types ());
    reference_factory->set_degree (2);

    PortableServer::ObjectId_var reference_id 
	= PortableServer::string_to_ObjectId ("ExternalizationReference_impl");

    poa->activate_object_with_id (*references_id, references_role_factory);
    poa->activate_object_with_id (*referencedby_id, referencedby_role_factory);
    poa->activate_object_with_id (*reference_id, reference_factory);

    mgr->activate ();

    if (trader)
    {
	CORBA::Object_var tr_obj;
#ifdef HAVE_EXCEPTIONS
	try {
#endif
	    tr_obj = orb -> resolve_initial_references("TradingService");
#ifdef HAVE_EXCEPTIONS
	} catch(CORBA::ORB::InvalidName&) {
	    cerr << "Can`t resolve Trading Service. Abort" << endl;
	    exit(1);
	}
#endif
	if (CORBA::is_nil(tr_obj))
	{
	    cerr << "Can`t resolve Trading Service. Abort" << endl;
	    exit(1);
	}
	CosTrading::Lookup_var trader_ = CosTrading::Lookup::_narrow (tr_obj);
	CosTrading::Register_var reg = trader_->register_if();
	if (CORBA::is_nil(reg))
	{
	    cerr << "Invalid Trading Service. Abort" << endl;
	    exit(1);
	}
	CORBA::Object_var obj = trader_->type_repos();
	CosTradingRepos::ServiceTypeRepository_var repo =
		CosTradingRepos::ServiceTypeRepository::_narrow(obj);
	if (CORBA::is_nil(repo))
	{
	    cerr << "Invalid Trading Service. Abort" << endl;
	    exit(1);
	}

	CORBA::Boolean role_type = false;
	CORBA::Boolean rel_type = false;
	CosTradingRepos::ServiceTypeRepository::SpecifiedServiceTypes all_;
	all_._d(CosTradingRepos::ServiceTypeRepository::all);
	CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq* list_of_type = 
		repo -> list_types(all_);
	for( CORBA::ULong i = 0;i < list_of_type->length();++i)
	{
	    CosTradingRepos::ServiceTypeRepository::TypeStruct* tmp_type =
		    repo -> describe_type((*list_of_type)[i]);
	    if (!strcmp("RoleFactory",(*list_of_type)[i]) &&
		!strcmp("IDL:omg.org/CosRelationships/RoleFactory:1.0",tmp_type->if_name))
		role_type = true;
	    if (!strcmp("RelationshipFactory",(*list_of_type)[i]) &&
		!strcmp("IDL:omg.org/CosRelationships/RelationshipFactory:1.0",tmp_type->if_name))
		rel_type = true;
	    delete tmp_type;
	    if (role_type && rel_type)
		break;
	}
	delete list_of_type;
	if (!role_type)
	{
	    CosTradingRepos::ServiceTypeRepository::IncarnationNumber i1;
	    {
		CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq super;
		super.length( 0 );
		CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
		props.length( 3 );
		props[0].name = CORBA::string_dup( "ObjectInterface" );
		props[0].value_type = CORBA::_tc_string;
		props[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
		props[1].name = CORBA::string_dup( "ImplEquivClass" );
		props[1].value_type = CORBA::_tc_string;
		props[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
		props[2].name = CORBA::string_dup( "ObjectImpl" );
		props[2].value_type = CORBA::_tc_string;
		props[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

		i1 = repo->add_type( "RoleFactory", "IDL:omg.org/CosRelationships/RoleFactory:1.0", props, super );
	    }
	}
	if (!rel_type)
	{
	    CosTradingRepos::ServiceTypeRepository::IncarnationNumber i2;
	    {
		CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq super;
		super.length( 0 );
		CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
		props.length( 3 );
		props[0].name = CORBA::string_dup( "ObjectInterface" );
		props[0].value_type = CORBA::_tc_string;
		props[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;
		props[1].name = CORBA::string_dup( "ImplEquivClass" );
		props[1].value_type = CORBA::_tc_string;
		props[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
		props[2].name = CORBA::string_dup( "ObjectImpl" );
		props[2].value_type = CORBA::_tc_string;
		props[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
    
		i2 = repo->add_type( "RelationshipFactory", "IDL:omg.org/CosRelationships/RelationshipFactory:1.0", props, super );
    	    }
	}
	CosTrading::OfferId id1;
	CosTrading::OfferId id2;
	CosTrading::OfferId id3;

	CosTrading::PropertySeq seq;
	seq.length(3);
	seq[0].name = CORBA::string_dup( "ObjectInterface" );
	seq[0].value <<= CORBA::Any::from_string( "::CosCompoundExternalization::Role", 0 );
	seq[1].name = CORBA::string_dup( "ImplEquivClass" );
	seq[1].value <<= CORBA::Any::from_string("", 0);
	seq[2].name = CORBA::string_dup( "ObjectImpl" );
	seq[2].value <<= CORBA::Any::from_string("::CosExternalizationReference::ReferencesRole", 0);

	id1 = reg->CORBA_CXX_PREFIX(export)(references_role_factory->_this(),
					    "RoleFactory", seq);

	seq.length(3);
	seq[0].name = CORBA::string_dup( "ObjectInterface" );
	seq[0].value <<= CORBA::Any::from_string( "::CosCompoundExternalization::Role", 0 );
	seq[1].name = CORBA::string_dup( "ImplEquivClass" );
	seq[1].value <<= CORBA::Any::from_string("", 0);
	seq[2].name = CORBA::string_dup( "ObjectImpl" );
	seq[2].value <<= CORBA::Any::from_string("::CosExternalizationReference::ReferencedByRole", 0);

        id2 = reg->CORBA_CXX_PREFIX(export)(referencedby_role_factory->_this(),
					    "RoleFactory", seq);

	seq.length(3);
	seq[0].name = CORBA::string_dup( "ObjectInterface" );
	seq[0].value <<= CORBA::Any::from_string( "::CosCompoundExternalization::Relationship", 0 );
	seq[1].name = CORBA::string_dup( "ImplEquivClass" );
	seq[1].value <<= CORBA::Any::from_string("", 0);
	seq[2].name = CORBA::string_dup( "ObjectImpl" );
	seq[2].value <<= CORBA::Any::from_string("::CosExternalizationReference::Relationship", 0);

	id3 = reg->CORBA_CXX_PREFIX(export)(reference_factory->_this(),
					    "RelationshipFactory", seq);
    }
    orb->run ();

    poa->destroy (TRUE, TRUE);

    return 0;
}

