/*
 *  LifeCycle Service for MICO
 *  Copyright (C) 1999 Dmitry V. Sedov
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 *  or to my private e-mail:
 *                 sedov@postman.simcb.ru
 */


#include <CORBA.h>
#include <mico/CosTradingRepos.h>
#include <mico/FactoryFinder_impl.h>
#include <string>

FactoryFinder_impl::FactoryFinder_impl()
#ifndef _WINDOWS
    : POA_CosLifeCycle::FactoryFinder ()
#endif
{
    orb_ = CORBA::ORB::_nil();
    trader_ = CosTrading::Lookup::_nil();
}

FactoryFinder_impl::FactoryFinder_impl(CORBA::ORB_ptr orb)
#ifndef _WINDOWS
    : POA_CosLifeCycle::FactoryFinder ()
#endif
{
    orb_ = CORBA::ORB::_duplicate(orb);
    CORBA::Object_var tr_obj;
#ifdef HAVE_EXCEPTIONS
    try {
#endif
	tr_obj = orb -> resolve_initial_references("TradingService");
#ifdef HAVE_EXCEPTIONS
    } catch(CORBA::ORB::InvalidName&) {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
#endif
    if (CORBA::is_nil(tr_obj))
    {
	cerr << "Can`t resolve Trading Service. Abort" << endl;
	exit(1);
    }
    trader_ = CosTrading::Lookup::_narrow (tr_obj);
}

FactoryFinder_impl::FactoryFinder_impl(CORBA::ORB_ptr orb, CosTrading::Lookup_ptr trader)
#ifndef _WINDOWS
    : POA_CosLifeCycle::FactoryFinder ()
#endif
{
    orb_ = CORBA::ORB::_duplicate(orb);
    trader_ = CosTrading::Lookup::_duplicate(trader);
}

FactoryFinder_impl::~FactoryFinder_impl()
{
}

CosLifeCycle::Factories*
FactoryFinder_impl::find_factories(const CosLifeCycle::Key& factory_key)
{
    if (CORBA::is_nil(trader_))
	return NULL;
    CosLifeCycle::Factories* result = new CosLifeCycle::Factories;

    CosTrading::TypeRepository_var tr_obj = trader_ -> type_repos();
    CosTradingRepos::ServiceTypeRepository_var type_repo =
	    CosTradingRepos::ServiceTypeRepository::_narrow(tr_obj);
    CosTradingRepos::ServiceTypeRepository::SpecifiedServiceTypes all_;
    all_._d(CosTradingRepos::ServiceTypeRepository::all);
    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq* list_of_type = 
	    type_repo -> list_types(all_);
    string filter;
    string type_;
    CORBA::UShort count=0;
    for(CORBA::ULong j = 0;j < factory_key.length();++j)
    {
	if (!strcmp(factory_key[j].kind,"object interface") &&
	    strlen(factory_key[j].id.in()))
	{
	    ++count;
	    if (filter.size())
	    {
		if (count>2)
		    filter += ")";
		filter += " and ";
	    }
	    filter += "'";
	    filter += factory_key[j].id.in();
	    filter += "' ~ ObjectInterface";
	}
	if (!strcmp(factory_key[j].kind,"implementation equivalence class") &&
	    strlen(factory_key[j].id.in()))
	{
	    ++count;
	    if (filter.size())
	    {
		if (count>2)
		    filter += ")";
		filter += " and ";
	    }
	    filter += "'";
	    filter += factory_key[j].id.in();
	    filter += "' ~ ImplEquivClass";
	}
	if (!strcmp(factory_key[j].kind,"object implementation") &&
	    strlen(factory_key[j].id.in()))
	{
	    ++count;
	    if (filter.size())
	    {
		if (count>2)
		    filter += ")";
		filter += " and ";
	    }
	    filter += "'";
	    filter += factory_key[j].id.in();
	    filter += "' ~ ObjectImpl";
	}
	if (!strcmp(factory_key[j].kind,"factory interface"))
	    type_ += factory_key[j].id.in();
    }
    if (count > 2)
    {
	string add_;
	for(CORBA::UShort i = 2;i < count;++i)
	    add_ += "(";
	add_ += filter;
	filter = add_;
    }
    for( CORBA::ULong i = 0;i < list_of_type->length();++i)
	if (!strcmp(type_.c_str(),(*list_of_type)[i]))
	{
	    CosTrading::Lookup::SpecifiedProps specifiedProps;
	    specifiedProps._default();
	    specifiedProps._d(CosTrading::Lookup::all);

	    // Get some pointers for the out parameters of the call.
	    CosTrading::OfferSeq* offerSeq_ptr = NULL;
	    CosTrading::OfferIterator* offerIterator_ptr = NULL;
	    CosTrading::PolicyNameSeq* policyNameSeq_ptr = NULL;

	    // An empty policy sequence
	    CosTrading::PolicySeq policySeq;
	    policySeq.length(0);

	    trader_->query ((*list_of_type)[i],
			    filter.c_str(),
			    "",
			    policySeq,
			    specifiedProps,
			    100,
			    offerSeq_ptr,
			    offerIterator_ptr,
			    policyNameSeq_ptr);

	    if (policyNameSeq_ptr) delete policyNameSeq_ptr;
	    // Check if an offer was made
	    if (offerSeq_ptr != NULL)
	    {
		// Insert the pointer into the out class
		CosTrading::OfferSeq_var offerSeq_var(offerSeq_ptr);
	  
		// We need at least one offer.
		CORBA::ULong len_ = (*result).length();
		(*result).length(len_+offerSeq_var->length());
		for (CORBA::ULong k = 0;k < offerSeq_var->length();++k)
		    // now we are all set to read from the sequence the result
		    (*result)[len_+k] = CORBA::Object::_duplicate(offerSeq_var[k].reference.in());

	    }
	    if (!CORBA::is_nil(offerIterator_ptr))
	    {
		CosTrading::OfferIterator_var offerIterator(offerIterator_ptr);
		CosTrading::OfferSeq* tmp_offers;
		if (!offerIterator->next_n(offerIterator->max_left(),tmp_offers))
		    continue;
		CORBA::ULong len_ = (*result).length();
		(*result).length(len_+(*tmp_offers).length());
		for(CORBA::ULong k = 0;k < (*tmp_offers).length();++k)
		    (*result)[len_+k] = CORBA::Object::_duplicate((*tmp_offers)[k].reference.in());
	    }
	}
    delete list_of_type;
    return result;
}

